#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
"""Table Creation/Deletion for Schema changes in the PowerVC 1.2.2 Release"""

from sqlalchemy import String, Table, MetaData, Column
from oslo_log import log as logging
from powervc_cinder import _

LOG = logging.getLogger(__name__)

STORAGE_NODE_COLUMNS = [Column('compression_supported', String(8))]
TABLE_COLUMN_UPDATES = [('storage_nodes', STORAGE_NODE_COLUMNS)]


def upgrade(migration_engine):
    """ Upgrades the Cinder DB Tables to include those for PowerVC 1.2.2 """
    metadata = MetaData(migration_engine)
    metadata.reflect(migration_engine)
    metadata.bind = migration_engine

    # Loop through each of the tables we need to add columns to
    for column_update in TABLE_COLUMN_UPDATES:
        table_name, table_columns = column_update
        try:
            # We need to add the additional columns to the given table
            atable = Table(table_name, metadata, autoload=True)
            for column in table_columns:
                # Check first to see if the Column exists on the Table already
                if atable._columns.get(column.name) is None:
                    atable.create_column(column.copy())
        except Exception as exc:
            info = dict(table=table_name, exc=exc)
            LOG.warn(_("Exception updating %(table)s: %(exc)s") % info)
            raise exc


def downgrade(migration_engine):
    """ Downgrades the Cinder DB Tables to remove those for PowerVC 1.2.2 """
    pass
