#
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from __future__ import print_function

import logging
import paramiko
import six

try:
    from scp import SCPClient
except ImportError:
    scp_installed = False
else:
    scp_installed = True

import powervc_cinder.k2aclient.openstack.common.procutils as procutils
from powervc_cinder.k2aclient.openstack.common import lockutils

LOG = logging.getLogger(__name__)

synchronized = lockutils.synchronized_with_prefix('k2a-')


# /powervc-discovery/powervc_discovery/registration/registrar_utilities.py
def _get_ssh_connection(ip, username, password, port=22, timeout=120):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.RejectPolicy())
    ssh.connect(ip, port=port,
                username=username,
                password=password,
                timeout=timeout)
    return ssh


@synchronized('ssh')
def sshftpget(
        ip,
        remotepath,
        localpath,
        username="padmin",
        password="passw0rd",
        suppress=False):
    try:
        # get a transport
        port = 22
        transport = paramiko.Transport((ip, port))

        # authorize
        transport.connect(username=username, password=password)

        sftp = paramiko.SFTPClient.from_transport(transport)

#         # Download
#         filepath = '/etc/passwd'
#         localpath = '/home/remotepasswd'
        sftp.get(remotepath, localpath)

#         # Upload
#         filepath = '/home/foo.jpg'
#         localpath = '/home/pony.jpg'
#         sftp.put(localpath, filepath)

        # close
        sftp.close()
        transport.close()

    except Exception as exc:
        if suppress is False:
            raise
        return ('', six.text_type(exc))


@synchronized('ssh')
def sshscpget(
        ip,
        remotepath,
        localpath,
        username="padmin",
        password="passw0rd",
        suppress=False):

    if not scp_installed:
        msg = [
            "# scp.py not installed",
            "git clone https://github.com/jbardin/scp.py.git",
            "cd scp.py"
            "python setup.py install"]
        raise Exception("\n" + "\n".join(msg))

    try:
        ssh = _get_ssh_connection(ip,
                                  username,
                                  password)

        # https://github.com/jbardin/scp.py
        # SCPCLient takes a paramiko transport as its only argument
        scp = SCPClient(ssh.get_transport())

#         scp.put('test.txt', 'test2.txt')
        scp.get(remotepath,
                local_path=localpath)

        ssh.close()
    except Exception as exc:
        if suppress is False:
            raise
        return ('', six.text_type(exc))


@synchronized('ssh')
def sshrun(
        ip,
        cmd,
        username="padmin",
        password="passw0rd",
        suppress=False):
    """Shared Helper Method to Run an Command thru SSH to the System"""
    try:
        ssh = _get_ssh_connection(ip,
                                  username,
                                  password)
        # Execute the given SSH command that was provided on input
        out, err = procutils.ssh_execute(
            ssh, cmd, check_exit_code=(suppress is False))
        # Make sure to close the SSH connection when we are done with it
        ssh.close()
        return (out, err)
    except Exception as exc:
        if suppress is False:
            raise
        return ('', six.text_type(exc))
