#
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Web Job interface."""

from eventlet import greenthread
from powervc_cinder.k2aclient import base
from powervc_cinder import _

import logging
LOG = logging.getLogger(__name__)


class JobManager(base.Manager):
    """Manage web jobs."""
    resource_class = None

    def getjob(self, resource, jobname, xa=None):
        return self._getjob(resource, jobname, xa=xa)

    def runjob(self, resource, jobrequest, xa=None):
        return self._runjob(resource, jobrequest, xa=xa)

    def readjob(self, job_id, xa=None):
        return self._readjob(job_id, xa=xa)

    def runandpolljob(
            self,
            jobhome,
            jobtype,
            jpc=None,
            maxretries=3,
            sleepbetweenretries=10,
            xa=None):

        retries = 0
        retry = True
        while retry:
            jrequest = self.api.web_job.getjob(jobhome,
                                               jobtype,
                                               xa=xa)

            if jpc:
                jrequest.job_parameters = jpc

            jresponse = self.api.web_job.runjob(jobhome,
                                                jrequest,
                                                xa=xa)
            jobid = jresponse.job_id
            k2respi = jresponse._k2resp
            retry = False
            while (jresponse.status == 'NOT_STARTED' or
                   jresponse.status == 'RUNNING' or
                   jresponse.status == 'FAILED_BEFORE_COMPLETION_RETRY'):

                if jresponse.status == 'FAILED_BEFORE_COMPLETION_RETRY':
                    retries += 1
                    if retries < maxretries:
                        msg = _("k2aclient:"
                                " during %(jobtype)s,"
                                " retry job,"
                                " job_id: >%(job_id)s<,"
                                " status: >%(status)s<,")
                        LOG.warn(msg % {"jobtype": jobtype,
                                        "job_id": jresponse.job_id,
                                        "status": jresponse.status, }
                                 )
                        greenthread.sleep(sleepbetweenretries)
                        retry = True
                    else:
                        msg = _("k2aclient:"
                                " during %(jobtype)s,"
                                " too many retries,"
                                " maximum number of retries: >%(maxretries)d<,"
                                " job_id: >%(job_id)s<,"
                                " status: >%(status)s<,")
                        LOG.error(msg % {"jobtype": jobtype,
                                         "maxretries": maxretries,
                                         "job_id": jresponse.job_id,
                                         "status": jresponse.status, })
                    break  # FAILED_BEFORE_COMPLETION_RETRY

                greenthread.sleep(1)
                jresponse = self.api.web_job.readjob(jresponse.job_id,
                                                     xa=xa)
            self.deletebyid(jobid, xa=xa)
        return k2respi, jresponse

#     def delete(self, job, child=None, xa=None):
#         """Delete the specified instance
#         """
#         return self._delete("uom", job,
#                             child=child, xa=xa)

    def deletebyid(self, job_id,
                   child_type=None, child_id=None, xa=None):
        """Delete the specified instance
        """
        return self._deletebyid("uom", "jobs", job_id,
                                child_type, child_id, xa=xa)
