# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
PowerVC Extension to the Cinder Host Manager
"""
from oslo_config import cfg

from cinder import db
from cinder import exception
from oslo_concurrency import lockutils
from oslo_log import log as logging
from cinder.scheduler import host_manager as hostmgr

from powervc_cinder import _

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
CONF.import_opt('scheduler_default_filters', 'cinder.scheduler.host_manager')
CONF.set_default('scheduler_default_filters',
                 ['AvailabilityZoneFilter',
                  'PowerVCCapacityFilter',
                  'CapabilitiesFilter'])


class PowerVCHostManager(hostmgr.HostManager):
    """ PowerVC Host Manager Extension """
    def __init__(self, *args, **kwargs):
        super(PowerVCHostManager, self).__init__(*args, **kwargs)

    @lockutils.synchronized('PowerVCHostManager', 'cinder-', external=True)
    def get_all_host_states(self, context):
        """
        The host_state_map in the HostManager isn't
        maintained properly during storage host registration/deregistration.
        Override get_all_host_states and add proper serialization.

        For example:
        {'192.168.1.100': HostState(), ...}
        """
        topic = CONF.volume_topic

        for host, host_state in self.host_state_map.items():
            try:
                db.service_get_by_host_and_topic(context,
                                                 host,
                                                 topic)
            except exception.ServiceNotFound:
                # The host has been deregistered
                LOG.debug(_("clean up host_state_map: %(host)s" %
                            {'host': host}))
                del self.host_state_map[host]
                continue

        s = super(PowerVCHostManager, self)
        hosts = s.get_all_host_states(context)

        return hosts
