# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_log import log as logging
from oslo_utils import importutils

from cinder import exception
from cinder.i18n import _, _LE
from cinder.zonemanager import fc_san_lookup_service as fc_service

LOG = logging.getLogger(__name__)


class PvcFCSanLookupService(fc_service.FCSanLookupService):
    """PowerVC Lookup Service.

    """

    def __init__(self, **kwargs):
        super(PvcFCSanLookupService, self).__init__(**kwargs)
        self.configuration = kwargs.get('configuration', None)

    def get_device_mapping_from_network(self, initiator_list, target_list):
        """Get device mapping from FC network.
        Gets a filtered list of initiator ports and target ports for each SAN
        available.

        Powervc - Override to return all initiators/targets back into
        the mappings, this is to avoid a double lookup on the SAN.
        The Zonemanager itself will handle the wwns lookups later
        during the zoning process.
        """
        # Initialize vendor specific implementation of  FCZoneDriver
        fabric_names = self.configuration.fc_fabric_names
        fabrics = [x.strip() for x in fabric_names.split(',')]
        device_map = {}
        for fabric in fabrics:
            fabric_map = {
                'initiator_port_wwn_list': initiator_list,
                'target_port_wwn_list': target_list}
            device_map[fabric] = fabric_map
        LOG.debug("Exit PowerVC-get_device_mapping_from_network. Returning "
                  "'device map': %s" % device_map)
        return device_map
