# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
Base Message Classes for all On-boarding related Messages
"""

import powervc_discovery.locale
_ = powervc_discovery.locale._

RSCTYPE_VM = 'PDI-B000001'
RSCTYPE_VOLUME = 'PDI-B000002'
RSCTYPE_VMS = 'PDI-B000003'
RSCTYPE_VOLUMES = 'PDI-B000004'

SYSTYPE_HOST = 'PDI-B000101'
SYSTYPE_STORAGE = 'PDI-B000102'

IMPORT_ACTION = 'PDI-B000301'
UNMANAGE_ACTION = 'PDI-B000302'

NO_HOST = 'PD-B000001'
NO_RESOURCES = 'PD-B000002'
NO_BOOT_VOLUME = 'PD-B000003'
BOOT_FOR_LOCAL = 'PD-B000004'
INVALID_RESOURCES = 'PD-B000005'
ACTION_FAILURE = 'PD-B000006'
IMPORT_COMPLETED1 = 'PD-B000007'
IMPORT_COMPLETED2 = 'PD-B000008'
IMPORT_COMPLETED3 = 'PD-B000009'
IMPORT_COMPLETED4 = 'PD-B000010'
IMPORT_ESCAPE = 'PD-B000011'
IMPORT_FAILED = 'PD-B000012'
IMPORT_NO_STORAGE = 'PD-B000013'
SERVICE_DOWN = 'PD-B000014'
ATTACHED_VOLUMES = 'PD-B000015'
ATTACHED_IMAGES = 'PD-B000016'
ATTACHED_BOTH = 'PD-B000017'
IMPORT_NETWORK_CHANGED = 'PD-B000018'
IMPORT_NETWORK_FAILED = 'PD-B000019'
NO_IMPORT_TASK = 'PD-B000020'
INVALID_PENDING_ACTIONS = 'PD-B000021'
IMPORT_ONLY_GENERIC = 'PD-B000022'
INVALID_RESOURCE_IDS = 'PD-B000023'
PREPARE_FINISHED1 = 'PD-B000024'
PREPARE_FINISHED2 = 'PD-B000025'


########################################################
# ###########  Base Importer Implementation  ###########
########################################################
class BaseImporterMessageBundle(object):

    def get_message_text(self, context, msgid, **msgparms):
        if msgid in _BASE_MESSAGES:
            msg = _BASE_MESSAGES.get(msgid) % msgparms
            # We only want to put the Message ID on if not internal
            if not msgid.startswith('PDI-'):
                msg = powervc_discovery.locale.KeyedMessage(msgid, msg)
            return msg
        msgtext = _("Unexpected MessageId %(msgid)s with Parameters: %(parm)s")
        return msgtext % {'msgid': msgid, 'parm': str(msgparms)}


_BASE_MESSAGES = {
    RSCTYPE_VM: _("virtual machine"),
    RSCTYPE_VOLUME: _("volume"),
    RSCTYPE_VMS: _("virtual machines"),
    RSCTYPE_VOLUMES: _("volumes"),
    SYSTYPE_HOST: _("host"),
    SYSTYPE_STORAGE: _("storage system"),
    IMPORT_ACTION: _("management"),
    UNMANAGE_ACTION: _("unmanage"),

    NO_HOST: _("Host %(hostname)s could not be found."),
    NO_RESOURCES: _("No %(rsctypes)s were specified on the request."),
    NO_BOOT_VOLUME:  _("The boot volume %(volid)s does not exist."),
    BOOT_FOR_LOCAL: _("The boot volume cannot be selected for local storage."),
    INVALID_RESOURCES: _("The following %(rsctypes)s specified do not "
                         "exist: %(rscids)s."),
    INVALID_RESOURCE_IDS: _("The resource identifier '%(rscid)s' "
                            "is not in a valid UUID format."),
    ACTION_FAILURE: _("The %(action)s operation failed for the following "
                      "%(rsctypes)s: %(rscids)s. Refer to the /var/log/"
                      "project/api.log file for more details."),
    IMPORT_ESCAPE: _("The management operation for %(rsctypes)s on "
                     "%(systype)s %(hostname)s did not complete. The "
                     "following error occurred: %(fault)s"),
    IMPORT_FAILED: _("The management operation failed for %(rsctype)s "
                     "%(rscname)s on %(systype)s %(hostname)s. The following "
                     "error occurred: %(fault)s"),
    IMPORT_COMPLETED1: _("The management operation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s completed.  %(success)s "
                         "were added to PowerVC."),
    IMPORT_COMPLETED2: _("The management operation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s completed.  %(success)s "
                         "were added to PowerVC and %(failed)s failed."),
    IMPORT_COMPLETED3: _("The management operation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s completed.  %(success)s "
                         "were added to PowerVC and %(nosupport)s were not "
                         "supported."),
    IMPORT_COMPLETED4: _("The management operation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s completed.  %(success)s "
                         "were added to PowerVC, %(nosupport)s were not "
                         "supported, and %(failed)s failed."),
    IMPORT_NO_STORAGE: _("Some virtual machines are not candidates for "
                         "management because mappings for attached volumes "
                         "could not be found on the storage providers managed "
                         "by PowerVC.  To manage these virtual machines with "
                         "PowerVC, first make sure the storage providers and "
                         "fabrics for this virtual machine are added to "
                         "PowerVC.  The following virtual machines failed: "
                         "%(vmnames)s."),
    PREPARE_FINISHED1: _("The management preparation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s finished.  %(success)s "
                         "were prepared."),
    PREPARE_FINISHED2: _("The management preparation for %(rsctypes)s on "
                         "%(systype)s %(hostname)s finished.  %(success)s "
                         "were prepared and %(failed)s failed."),
    SERVICE_DOWN: _("Unable to contact the openstack-%(service)s "
                    "service to retrieve the list of resources."),
    ATTACHED_VOLUMES: _("The following volumes cannot be unmanaged since "
                        "they are attached to virtual machines: %(attached_"
                        "volumes)s.  Unmanage the virtual machines the "
                        "volumes are attached to and then try unmanaging "
                        "the volumes again."),
    ATTACHED_IMAGES: _("The following volumes cannot be unmanaged since they "
                       "are used by images: %(image_volumes)s.  Remove the "
                       "images and then try unmanaging the volumes again."),
    ATTACHED_BOTH: _("The following volumes cannot be unmanaged since they "
                     "are attached to virtual machines: %(attached_volumes"
                     ")s.  The following volumes cannot be unmanaged since "
                     "they are used by images: %(image_volumes)s.  Remove "
                     "the images and unmanage the virtual machines the "
                     "volumes are attached to and then try unmanaging the "
                     "volumes again."),
    IMPORT_NETWORK_CHANGED: _("PowerVC is now managing virtual machine "
                              "'%(vmname)s' on network '%(networkname)s'.  "
                              "The network and its peer networks on the same "
                              "virtual switch and VLAN have been changed so "
                              "that they now use the Shared Ethernet Adapter "
                              "that the virtual machine is using: "
                              "'%(seaname)s'.  This may be changed by "
                              "modifying the Shared Ethernet Adapter mapping "
                              "in the Network."),
    IMPORT_NETWORK_FAILED: _("Unable to bring virtual machine %(vmname)s "
                             "under PowerVC management due to an error "
                             "while configuring the network. For more "
                             "information, see the logs."),
    NO_IMPORT_TASK: _("On-boarding task %(taskid)s could not be found."),
    INVALID_PENDING_ACTIONS: _("The imported virtual machine %(instuuid)s "
                               "cannot be %(action)sd until the boot volume "
                               "is selected."),
    IMPORT_ONLY_GENERIC: _("The virtual machines are not candidates for "
                           "management because only pluggable storage "
                           "providers are managed by PowerVC.  Managing "
                           "existing virtual machines is not supported when "
                           "using pluggable storage providers.")
}
