# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Base Classes for all Registration-related Activity"""

import sys

from powervc_nova.objects.network import dom as network_dom
from powervc_discovery.registration import registrar_utilities as utils
from powervc_discovery.registration.compute import host_registrar


########################################################
# ########  PowerVM Registrar Implementation  ##########
########################################################
class PowerVMHostRegistrar(host_registrar.HostRegistrar):
    """Abstract Base Class to handle Host Registration"""

    def __init__(self, context, **kwargs):
        """Constructor for Abstract Class for Host Registration"""
        super(PowerVMHostRegistrar, self).__init__(context, **kwargs)

    def get_required_attributes(self):
        """Sub-classes needs to override to do the Host Registration"""
        raise NotImplementedError()

    #############################################
    #  Implementing ServiceRegistrar Interface  #
    #############################################
    def deregister(self, force=False, uninstall=False, soft=False):
        """Registrar Method to delete a System Registration"""
        # Call the Parent Class to do the actual Removal of the Host
        super(PowerVMHostRegistrar, self).deregister(force, uninstall, soft)
        # Invoke the Network Method to Cleanup the State/Unused Ports
        netfact = network_dom.NetworkAssociationFactory.get_factory()
        netfact.cleanup_unused_ports(self.context)

    def _get_service_param_mapping(self):
        """Provides the mapping of Properties to be used by the Registrar"""
        mapping = super(PowerVMHostRegistrar,
                        self)._get_service_param_mapping()
        mapping['powervm_mgr_type'] = 'manager_type'
        return mapping


########################################################
#  ########  Registrar Factory Implementation  #########
########################################################
def get_registrar(context, **kwargs):
    """Factory Method to retrieve an instance of the proper Registrar"""
    if kwargs.get('create'):
        powervm_mgr_type = 'hmc' if 'hmc_uuids' in kwargs else 'pvm'
    # If this isn't a create, we need to get it from the CONF file
    else:
        conf_file = '/etc/nova/nova-' + kwargs.get('host_name') + '.conf'
        mgr_type = utils.read_config_file(conf_file).get('powervm_mgr_type')
        powervm_mgr_type = mgr_type if mgr_type else 'hmc'
    # Attempt to dynamically load the corresponding module for the host type
    regmodule = (__package__ + '.' + powervm_mgr_type + '_powervm_registrar')
    __import__(regmodule)
    return getattr(sys.modules[regmodule], 'get_registrar')(context, **kwargs)
