# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
Base Classes for all Registration-related Messages
"""

import powervc_discovery.locale
_ = powervc_discovery.locale._

REG_TYPE_HOST = 'PDI-A000001'
REG_TYPE_STORAGE = 'PDI-A000002'
REG_TYPE_HMC = 'PDI-A000003'
REG_TYPE_FABRIC = 'PDI-A000004'

SYSTEM_TYPE_SYSTEM = 'PDI-A000101'
SYSTEM_TYPE_VIOS = 'PDI-A000102'
SYSTEM_TYPE_HMC = 'PDI-A000103'
SYSTEM_TYPE_SVC = 'PDI-A000104'
SYSTEM_TYPE_EMC_VNX = 'PDI-A000105'
SYSTEM_TYPE_BROCADE = 'PDI-A000106'
SYSTEM_TYPE_KVM = 'PDI-A000107'
SYSTEM_TYPE_POWERKVM = 'PDI-A000108'
SYSTEM_TYPE_NFS = 'PDI-A000109'
SYSTEM_TYPE_XIV = 'PDI-A000110'
SYSTEM_TYPE_CISCO = 'PDI-A000111'
SYSTEM_TYPE_GENERIC_STG = 'PDI-A000112'
SYSTEM_TYPE_GENERIC_FAB = 'PDI-A000113'
SYSTEM_TYPE_HITACHI = 'PDI-A000114'
SYSTEM_TYPE_EMC_VMAX = 'PDI-A000115'
SYSTEM_TYPE_NOVALINK = 'PDI-A000116'
SYSTEM_TYPE_DS8K = 'PDI-A000117'

RSC_TYPE_SERVER = 'PDI-A000201'
RSC_TYPE_VOLUME = 'PDI-A000202'
RSC_TYPE_SERVERS = 'PDI-A000203'
RSC_TYPE_VOLUMES = 'PDI-A000204'

EDITED_ACTION = 'PDI-A000301'
REMOVED_ACTION = 'PDI-A000302'
EDITING_ACTION = 'PDI-A000303'
REMOVING_ACTION = 'PDI-A000304'

PRIVKEY_NOTFOUND = 'PD-A000001'
PRIVKEY_ERROR = 'PD-A000002'
PASSWORD_ENCRYPT_ERROR = 'PD-A000003'
MISSING_ATTRIBUTE = 'PD-A000004'
IMMUTABLE_ATTRIBUTE = 'PD-A000005'
INVALID_ATTRIBUTE_VALUE = 'PD-A000006'
INVALID_CHARACTERS = 'PD-A000007'
SYSTEM_EXISTS = 'PD-A000008'
SYSTEM_NOTFOUND = 'PD-A000009'
SWITCHING_SYSTEMS = 'PD-A000010'
INVALID_HOST_TYPE = 'PD-A000011'
STORAGE_NOTFOUND = 'PD-A000012'
SINGLE_STORAGE = 'PD-A000013'
FABRIC_LIMIT = 'PD-A000014'
REMOVE_COMPUTE = 'PD-A000015'
REMOVE_ATTACHED_VOLUMES = 'PD-A000016'
REMOVE_IMAGE_VOLUMES = 'PD-A000017'
SSP_REMOVE_INTERNAL = 'PD-A000018'
TASKS_INPROGRESS = 'PD-A000019'
HMC_NOTFOUND = 'PD-A000020'
SINGLE_HMC = 'PD-A000021'
NONHMC_ENVIRONMENT = 'PD-A000022'
SSP_REMOVE_IMGVOLS = 'PD-A000023'
NO_SOFTWARE_UPDATE = 'PD-A000024'
NO_SOFTWARE_VERSION = 'PD-A000025'
SOFTWARE_UPTODATE = 'PD-A000026'
TWO_FABRIC_TYPES = 'PD-A000027'
DUP_DISP_NAMES = 'PD-A000028'
USER_NEEDS_PWD = 'PD-A000029'
NO_NAVISECCLI = 'PD-A000030'
NO_PPC_NAVISECCLI = 'PD-A000031'
NO_HORCM = 'PD-A000032'
HORCM_STOPPED = 'PD-A000033'
WRITING_ERROR = 'PD-A000034'
CREATE_CONF_ERROR = 'PD-A000035'
HMC_NO_HOST_CONNECTION = 'PD-A000036'
PVM_NOT_HMC = 'PD-A000037'
HMC_HOST_IN_RECOVERY = 'PD-A000038'

AUTHENTICATION_ERROR = 'PD-A000101'
AUTHENTICATION_ERROR_SSH = 'PD-A000102'
CONNECTION_TIMEOUT = 'PD-A000111'
CONNECTION_TIMEOUT_HMC = 'PD-A000112'
CONNECTION_TIMEOUT_SSH = 'PD-A000113'
CONNECTION_FAILED = 'PD-A000121'
CONNECTION_FAILED_HMC = 'PD-A000122'
CONNECTION_FAILED_SSH = 'PD-A000123'
CONNECTION_FIREWALL = 'PD-A000131'
CONNECTION_NOTKNOWN = 'PD-A00132'
CONNECTION_NOROUTE = 'PD-A000133'
CONNECTION_VMAXPORT = 'PD-A000134'
SERVICE_TIMEOUT = 'PD-A000141'
INVALID_CERTIFICATE = 'PDI-A000501'
CERTIFICATE_EXPIRED = 'PD-A000201'
INVALID_SYSTEM = 'PD-A000202'
INVALID_SVC_PRIVILEGES = 'PD-A000203'
INVALID_HMC_MANAGED = 'PD-A000204'
INVALID_SEA_CONFIG = 'PD-A000205'
INVALID_MISSING_SEA = 'PD-A000206'
INVALID_RMC_NO_SEA = 'PD-A000207'
INVALID_RMC_DOWN = 'PD-A000208'
INVALID_VIOS_DB = 'PD-A000209'
INVALID_VIOS_SSH = 'PD-A000210'
INVALID_POOL_ERROR = 'PD-A000211'
INVALID_IMAGE_PATH = 'PD-A000212'
STORAGE_DISPLAY_LEN = 'PD-A000213'
INVALID_SVC_ISCSI = 'PD-A000214'
INVALID_SYSTEM_VMAX = 'PD-A000215'
VMAX_NOT_MANAGED = 'PD-A000216'
NO_HOST_ON_HMC = 'PD-A000221'
NO_CLUSTER_ON_HMC = 'PD-A000222'
NO_VIOS_ON_HOST = 'PD-A000223'
INVALID_EMC_NOT_MANAGED = 'PD-A000224'
NO_HOST_MODES_RETURNED_K2 = 'PD-A000225'
NO_HOST_MODES_P6_OR_P7 = 'PD-A000226'
NO_HOST_FOUND_ON_HMC_FOR_MODES = 'PD-A000227'
MISMATCH_HOST_KEY = 'PD-A000529'
MISMATCH_HOST_KEY_I = 'PDI-A000529'
UNKNOWN_HOST_KEY = 'PDI-A000530'
SWITCH_IS_NOT_PRINCIPAL = 'PD-A000251'
SWITCH_IS_NOT_ZONING = 'PD-A000252'
SWITCH_PROPERTY_MISSING = 'PD-A000253'
KVM_INVALID_RHEL_SYSTEM = 'PD-A000301'
KVM_INVALID_X86_SYSTEM = 'PD-A000302'
KVM_INVALID_EPHEMERAL = 'PD-A000303'
HOST_ALREADY_MANAGED = 'PD-A000304'
HOST_OTHER_MANAGED = 'PD-A000305'
HOST_TRANSFER_STARTED = 'PD-A000306'
HOST_TRANSFER_FAILED = 'PD-A000307'
HOST_INSTALL_STARTED1 = 'PD-A000308'
HOST_INSTALL_STARTED2 = 'PD-A000309'
HOST_INSTALL_DONE1 = 'PD-A000310'
HOST_INSTALL_DONE2 = 'PD-A000311'
HOST_INSTALL_FAILED = 'PD-A000312'
HOST_UNINSTALL_FAILED = 'PD-A000313'
HOST_CONFIGURE_FAILED = 'PD-A000314'
HOST_RECONFIGURE_FAILED = 'PD-A000315'
HOST_UNCONFIGURE_FAILED = 'PD-A000316'
HOST_SOFTWARE_UPDATING = 'PD-A000317'
KVM_LIBVIRT_UNAVAILABLE = 'PD-A000331'
HOST_INVALID_TIME_SKEW = 'PD-A000332'
HOST_CANNOT_CONNECT_MGMT = 'PD-A000333'
HOST_NON_ROOT_USER = 'PD-A000334'
HOST_NO_SPACE = 'PD-A000335'
HOST_INVALID_HOSTNAME = 'PD-A000336'
HOST_SCP_NOT_FOUND = 'PD-A000337'
KVM_ABSOLUTE_PATH = 'PD-A000338'
HOST_NEED_SUDOER = 'PD-A000339'
HOST_SUDOER_NOPASS = 'PD-A000340'
HOST_ADD_INPROGRESS = 'PD-A000330'
HOST_DEL_INPROGRESS = 'PD-A000331'
HOST_HMC_FORCE_SWITCH = 'PD-A000341'
HOST_PVM_NOT_MASTER = 'PD-A000342'
HOST_PVM_FORCE_SWITCH = 'PD-A000343'
HOST_HMC_NOT_MASTER = 'PD-A000344'
HOST_HMC_NOT_MASTER2 = 'PD-A000348'
HOST_PVM_SWITCH_ERROR = 'PD-A000349'
HOST_PVM_NO_SSP = 'PD-A000350'
HOST_PVM_NO_SPP = 'PD-A000351'
HOST_PVM_MIN_VERISON = 'PD-A000352'

NFS_INVALID_IP = 'PD-A000401'
NFS_INVALID_MOUNT_OPTIONS = 'PD-A000402'
NFS_INVALID_SHARE = 'PD-A000403'
NFS_UNKNOWN_MOUNT_ERROR = 'PD-A000404'
PVM_NEED_PVMADMIN = 'PD-A000451'
UNKNOWN_ERROR = 'PD-A001001'


########################################################
#            Base Registrar Implementation             #
########################################################
class BaseRegistrarMessageBundle(object):

    def get_message_text(self, context, msgid, **msgparms):
        if msgid in _BASE_MESSAGES:
            msg = _BASE_MESSAGES.get(msgid) % msgparms
            # We only want to put the Message ID on if not internal
            if not msgid.startswith('PDI-'):
                msg = powervc_discovery.locale.KeyedMessage(msgid, msg)
            return msg
        msgtext = _("Unexpected MessageId %(msgid)s with Parameters: %(parm)s")
        return msgtext % {'msgid': msgid, 'parm': str(msgparms)}


_BASE_MESSAGES = {
    REG_TYPE_HOST: _("host"),
    REG_TYPE_STORAGE: _("storage system"),
    REG_TYPE_HMC: _("Hardware Management Console"),
    REG_TYPE_FABRIC: _("fabric"),

    SYSTEM_TYPE_SYSTEM: _("System"),
    SYSTEM_TYPE_VIOS: _("Virtual I/O Server"),
    SYSTEM_TYPE_HMC: _("Hardware Management Console"),
    SYSTEM_TYPE_NOVALINK: _("PowerVM NovaLink OS"),
    SYSTEM_TYPE_SVC: _("SAN Volume Controller"),
    SYSTEM_TYPE_EMC_VMAX: _("EMC VMAX SMI-S Provider"),
    SYSTEM_TYPE_EMC_VNX: _("EMC VNX Storage"),
    SYSTEM_TYPE_BROCADE: _("Brocade Switch"),
    SYSTEM_TYPE_KVM: _("Kernel-based Virtual Machine"),
    SYSTEM_TYPE_POWERKVM: _("IBM PowerKVM Host OS"),
    SYSTEM_TYPE_NFS: _("Network File System"),
    SYSTEM_TYPE_XIV: _("IBM XIV"),
    SYSTEM_TYPE_DS8K: _("IBM DS8000"),
    SYSTEM_TYPE_CISCO: _("CISCO Switch"),
    SYSTEM_TYPE_HITACHI: _("Hitachi Storage"),
    SYSTEM_TYPE_GENERIC_STG: _("pluggable storage provider"),
    SYSTEM_TYPE_GENERIC_FAB: _("pluggable fabric"),

    RSC_TYPE_SERVER: _("virtual machine"),
    RSC_TYPE_VOLUME: _("volume"),
    RSC_TYPE_SERVERS: _("virtual machines"),
    RSC_TYPE_VOLUMES: _("volumes"),

    EDITED_ACTION: _("edited"),
    REMOVED_ACTION: _("removed"),
    EDITING_ACTION: _("editing"),
    REMOVING_ACTION: _("removing"),

    PRIVKEY_NOTFOUND: _("The private key file '%(privkey)s' was not "
                        "found on the management host."),
    PRIVKEY_ERROR: _("The private key could not be loaded.  Specify a "
                     "private key in the correct PEM format and try again."),
    PASSWORD_ENCRYPT_ERROR: _("The password could not be encrypted by "
                              "the encryption command."),
    MISSING_ATTRIBUTE: _("The %(attr)s attribute is required."),
    IMMUTABLE_ATTRIBUTE: _("The %(attr)s attribute is not "
                           "allowed to be updated."),
    INVALID_ATTRIBUTE_VALUE: _("The %(attr)s attribute is set to an invalid "
                               "value."),
    INVALID_CHARACTERS: _("The name of the %(regtype)s can include only "
                          "letters, numbers, dashes (-), or underscores (_)."),
    SYSTEM_EXISTS: _("The %(regtype)s you are trying to register "
                     "already exists with the name '%(hostname)s'."),
    SYSTEM_NOTFOUND: _("The %(regtype)s %(hostname)s could not be found."),
    SWITCHING_SYSTEMS: _("The IP address or hostname specified is "
                         "for a different %(systype)s."),
    INVALID_HOST_TYPE: _("The '%(hosttype)s' type of %(regtype)s "
                         "is not supported."),
    STORAGE_NOTFOUND: _("Could not find storage system '%(hostname)s'."),
    SINGLE_STORAGE: _("Only one storage system can be registered at a time."),
    FABRIC_LIMIT: _("A maximum of 26 FC fabrics may be registered at any "
                    "given time."),
    REMOVE_COMPUTE: _("You must remove all hosts before you can remove "
                      "this storage system."),
    REMOVE_ATTACHED_VOLUMES: _("The storage system cannot be removed while "
                               "the following volumes are attached to virtual "
                               "machines: %(volumes)s.  Remove the host or "
                               "unmanage the virtual machines the volumes are "
                               "attached to and then try removing the storage "
                               "system."),
    REMOVE_IMAGE_VOLUMES: _("The storage system cannot be removed while the "
                            "following volumes are used by images: "
                            "%(volumes)s.  Remove the images and then try "
                            "removing the storage system."),
    SSP_REMOVE_INTERNAL: _("Storage Providers for Power Shared Storage Pools "
                           "can only be removed when all Hosts using the "
                           "Storage are removed."),
    TASKS_INPROGRESS: _("The %(regtype)s cannot be %(action)s while tasks "
                        "are in-progress.  Wait for the %(rsctype)s tasks to "
                        "finish and try %(actioning)s again.  The following "
                        "%(rsctypes)s had tasks in-progress: %(rscnames)s."),
    HMC_NOTFOUND: _("Could not find hardware "
                    "management console '%(hmcname)s'."),
    SINGLE_HMC: _("Only one hardware management console may be "
                  "specified when registering a host."),
    NONHMC_ENVIRONMENT: _("Management System is not a supported environment "
                          "for HMC-managed hosts"),
    SSP_REMOVE_IMGVOLS: _("The host %(hostname)s cannot be removed while "
                          "Power Shared Storage Pools have volumes used by "
                          "images.  Remove the images using volumes on "
                          "storage system %(providername)s and then try "
                          "removing the host."),
    NO_SOFTWARE_UPDATE: _("Updating the PowerVC software is not supported "
                          "on %(systype)s %(hostname)s."),
    NO_SOFTWARE_VERSION: _("The %(version)s version for the PowerVC software "
                           "does not exist on the management system."),
    SOFTWARE_UPTODATE: _("The PowerVC software on %(systype)s %(hostname)s "
                         "is already at version %(version)s."),
    TWO_FABRIC_TYPES: _("The %(systype)s could not be added to PowerVC.  "
                        "There is already a %(systype2)s registered with "
                        "PowerVC.  Only one type of fabric can be registered "
                        "at a time."),
    DUP_DISP_NAMES: _("There is already a %(regtype)s with a display name of "
                      "'%(dispname)s'.  Only one %(regtype)s can have a given "
                      "display name.  Choose another name and try again."),
    USER_NEEDS_PWD: _("The user id for the %(systype)s cannot be changed "
                      "without also specifying the password for that user."),
    NO_NAVISECCLI: _("The EMC Navisphere CLI is not installed on the "
                     "management system.  Please install the NaviCLI-Linux "
                     "RPM and try adding the storage system again."),
    NO_PPC_NAVISECCLI: _("The EMC Navisphere CLI is not supported on Linux "
                         "for IBM Power.  Please use a Linux for x86_64 "
                         "management system to manage EMC VNX storage."),
    NO_HORCM: _("Hitachi Open Remote Copy Manager is not installed "
                "on the management system. Please install HORCM "
                "and try adding the storage system again"),
    HORCM_STOPPED: _("Could not log into the RAID manager. Verify that "
                     "HORCM is up and running"),
    WRITING_ERROR: _("There was a problem writing to the horcm%(port)s.conf "
                     "file."),
    CREATE_CONF_ERROR: _("There was a problem creating the horcm%(port)s.conf "
                         "file"),
    HMC_NO_HOST_CONNECTION: _("The host %(hostname)s could not be added "
                              "because the Hardware Management Console (HMC) "
                              "does not have a connection to it."),
    PVM_NOT_HMC: _("The system %(hmcname)s could not be added as a Hardware "
                   "Management Console (HMC) because it is a PowerVM "
                   "NovaLink OS."),
    HMC_HOST_IN_RECOVERY: _("The host %(hostname)s could not be added because "
                            "the host is in recovery state on the Hardware "
                            "Management Console (HMC).  Take action on the "
                            "HMC to get the host out of recovery state and "
                            "try the operation again."),

    AUTHENTICATION_ERROR: _("Invalid user name or credentials specified "
                            "for the %(systype)s."),
    AUTHENTICATION_ERROR_SSH: _("Invalid user name or credentials specified "
                                "for the %(systype)s.  Verify the user and "
                                "credentials specified are correct and the "
                                "user is permitted to log in to the SSH "
                                "server."),
    CONNECTION_TIMEOUT: _("Could not connect to the %(systype)s in "
                          "the time allowed."),
    CONNECTION_TIMEOUT_HMC: _("Could not connect to the Hardware Management "
                              "Console (HMC) in the time allowed.  Verify "
                              "that the HMC is running and accessible, its "
                              "firewall allows incoming access on port 12443, "
                              "and the HMC version is V8R8.2.0 or later."),
    CONNECTION_TIMEOUT_SSH: _("Could not connect to the %(systype)s in the "
                              "time allowed.  Verify that the %(systype)s is "
                              "running and accessible, its firewall allows "
                              "incoming access on port 22, and the SSH server "
                              "is running and enabled."),
    CONNECTION_FAILED: _("Connection failed to the %(systype)s."),
    CONNECTION_FAILED_HMC: _("Connection failed to the Hardware Management "
                             "Console (HMC).  Verify that the HMC is running "
                             "and accessible, its firewall allows incoming "
                             "access on port 12443, and the HMC version is "
                             "V8R8.2.0 or later."),
    CONNECTION_FAILED_SSH: _("Connection failed to the %(systype)s.  Verify "
                             "that the %(systype)s is running and accessible, "
                             "its firewall allows incoming access on port "
                             "22, and the SSH server is running and enabled."),
    CONNECTION_FIREWALL: _("A firewall might be preventing you from "
                           "connecting to the %(systype)s."),
    CONNECTION_NOTKNOWN: _("An invalid IP address or hostname specified for "
                           "the %(systype)s."),
    CONNECTION_NOROUTE: _("There is no route to the IP address or host name "
                          "specified for the %(systype)s."),
    CONNECTION_VMAXPORT: _("Connection failed to the EMC VMAX SMI-S Provider "
                           "due to a protocol error.  This is most likely due "
                           "to the HTTP port number being specified rather "
                           "than the HTTPS port number.  Only HTTPS is "
                           "allowed for communicating with the SMI-S "
                           "Provider."),
    SERVICE_TIMEOUT: _("The %(service)s service did not "
                       "start in the time allowed."),
    INVALID_CERTIFICATE: _("The certificate for the %(systype)s is "
                           "untrusted or invalid."),
    CERTIFICATE_EXPIRED: _("The certificate on the %(systype)s has expired.  "
                           "Update the certificate and try again."),
    INVALID_SYSTEM: _("The specified IP address or host name does not belong "
                      "to a %(systype)s."),
    INVALID_SVC_PRIVILEGES: _("The user specified for the SAN Volume "
                              "Controller does not have sufficient privileges "
                              "for PowerVC to manage.  Please verify that the "
                              "user specified is part of an Administrator "
                              "role."),
    INVALID_HMC_MANAGED: _("The host is managed by a hardware management "
                           "console. Hosts must be managed by the Integrated "
                           "Virtualization Manager."),
    INVALID_SEA_CONFIG: _("The host must have a single available Shared "
                          "Ethernet Adapter."),
    INVALID_MISSING_SEA: _("The host must have at least one available Shared "
                           "Ethernet Adapter."),
    INVALID_RMC_NO_SEA: _("No Shared Ethernet Adapters were discovered.  This "
                          "may be due to an RMC connection to the Virtual IO "
                          "Server partition being down.  Please verify that "
                          "the system has a VIOS with at least one Shared "
                          "Ethernet Adapter that also has an active RMC "
                          "connection and try again."),
    INVALID_RMC_DOWN: _("The host could not be added because there were no "
                        "active RMC connections to any of the Virtual I/O "
                        "Servers.  Verify the host has at least one VIOS with "
                        "an active RMC connection and try the operation "
                        "again."),
    INVALID_VIOS_DB: _("The Hardware Management Console could not connect to "
                       "the database on the Virtual I/O Server.  Verify that "
                       "the Hardware Management Console can connect to all "
                       "Virtual I/O Servers for the Host and the vio_daemon "
                       "service is running on the Virtual I/O Servers."),
    INVALID_VIOS_SSH: _("The ioscli command on the Virtual I/O Server could "
                        "not be found.  Verify the 'PermitUserEnvironment' "
                        "property is set to 'yes' in the /etc/ssh/sshd_config "
                        "file on the VIOS and restart the SSH server."),
    INVALID_POOL_ERROR: _("The volume pool you specified does not exist."),
    INVALID_IMAGE_PATH: _("The %(imgdir)s directory to store image files does "
                          "not exist on the Virtual I/O Server.  Please "
                          "verify the path exists and try again."),
    STORAGE_DISPLAY_LEN: _("Display Name for a Storage Provider "
                           "may be at most 247 characters long."),
    INVALID_SVC_ISCSI: _("The %(systype)s is not configured to support "
                         "the iSCSI protocol.  Configure the %(systype)s "
                         "for iSCSI and try adding the %(regtype)s again."),
    INVALID_SYSTEM_VMAX: _("The %(systype)s is not "
                           "managing any EMC VMAX storage."),
    VMAX_NOT_MANAGED: _("The specified EMC VMAX storage "
                        "is not managed by the %(systype)s."),
    NO_HOST_ON_HMC: _("The host %(hostname)s does not exist on the "
                      "Hardware Management Console."),
    NO_CLUSTER_ON_HMC: _("The cluster %(hostname)s does not exist on the "
                         "Hardware Management Console."),
    NO_VIOS_ON_HOST: _("The host must have a Virtual I/O Server "
                       "configured and running."),
    INVALID_EMC_NOT_MANAGED: _("The EMC storage controller IP address "
                               "specified is not managed by the EMC SMI-S "
                               "Provider."),
    NO_HOST_MODES_RETURNED_K2: _("No supported processor modes were returned "
                                 "for host %(hostname)s. Cannot determine if "
                                 "the host is a Power 6, Power 7 or Power 8 "
                                 "processor type."),
    NO_HOST_MODES_P6_OR_P7: _("No supported processor modes were found for "
                              "host %(hostname)s. Only Power 6, Power 7 and "
                              "Power 8 hosts are supported."),
    NO_HOST_FOUND_ON_HMC_FOR_MODES: _("Host %(hostname)s was not found when "
                                      "trying to get the host supported "
                                      "process modes. Cannot determine if "
                                      "host is Power 6 or Power 7 processor "
                                      "type."),
    MISMATCH_HOST_KEY: _("The host key for the %(systype)s does not match the "
                         "existing key found in %(known_hosts)s."),
    MISMATCH_HOST_KEY_I: _("The host key for the %(systype)s does not match "
                           "the existing key found in %(known_hosts)s."),
    UNKNOWN_HOST_KEY: _("The authenticity of the %(systype)s "
                        "cannot be verified."),
    UNKNOWN_ERROR: _("Could not access the %(systype)s for an unknown reason. "
                     "Refer to the %(log)s file for more details."),
    SWITCH_IS_NOT_PRINCIPAL: _("The switch is not a principal switch, the "
                               "current role is %(current)s"),
    SWITCH_IS_NOT_ZONING: _("Zoning is not enabled on the switch, the current "
                            "zoning is %(current)s"),
    SWITCH_PROPERTY_MISSING: _("The following properties could not be found: "
                               "%(missing)s."),
    KVM_INVALID_RHEL_SYSTEM: _("The IP address or hostname that you specified "
                               "is not a Red Hat Enterprise Linux Server."),
    KVM_INVALID_X86_SYSTEM: _("The IP address or hostname that you specified "
                              "is not an x86_64 Server."),
    KVM_INVALID_EPHEMERAL: _("The Ephemeral Disk Path %(path)s is either "
                             "inaccessible or invalid."),
    HOST_ALREADY_MANAGED: _("The %(systype)s is already being managed by the "
                            "PowerVC management system at %(ip)s.  Only one "
                            "management system can manage a host at a "
                            "given time.  Remove the host from the other "
                            "management system or request it to be removed "
                            "from PowerVC management."),
    HOST_OTHER_MANAGED: _("The %(systype)s is already being managed by another"
                          " management system  at %(ip)s.  Only one management"
                          " system can manage a host at a given time.  Stop "
                          "managing the host with the other management "
                          "system and try again."),
    HOST_TRANSFER_STARTED: _("The PowerVC image transfer to the %(systype)s "
                             "%(hostname)s has started.  PowerVC install will "
                             "begin after the transfer completes."),
    HOST_TRANSFER_FAILED: _("The PowerVC image transfer to the %(systype)s "
                            "%(hostname)s failed.  Refer to the "
                            "/var/log/nova/nova-api.log on the management "
                            "system for more details."),
    HOST_INSTALL_STARTED1: _("The PowerVC install has started on the "
                             "%(systype)s %(hostname)s.  The host will be "
                             "added to the management system after the "
                             "install completes."),
    HOST_INSTALL_STARTED2: _("The PowerVC update has started on the "
                             "%(systype)s %(hostname)s.  The host will not "
                             "be available from the management system until "
                             "the update completes."),
    HOST_INSTALL_DONE1: _("The PowerVC install on the %(systype)s "
                          "%(hostname)s has completed.  The host will "
                          "now be added to the management system."),
    HOST_INSTALL_DONE2: _("The PowerVC update on the %(systype)s "
                          "%(hostname)s has completed.  The host is "
                          "now available on the management system."),
    HOST_INSTALL_FAILED: _("The PowerVC install on the %(systype)s "
                           "%(hostname)s failed.  Refer to the corresponding "
                           "/opt/ibm/powervc/log/powervc_install_{timestamp}"
                           ".log file on the host for more details."),
    HOST_UNINSTALL_FAILED: _("The PowerVC uninstall on the %(systype)s "
                             "%(hostname)s failed.  Refer to the "
                             "/var/log/powervc-uninstall.log file on "
                             "the host for more details."),
    HOST_CONFIGURE_FAILED: _("Configuring of the %(systype)s %(hostname)s "
                             "failed.  Refer to the /var/log/nova/nova-api.log"
                             " on the management system for more details."),
    HOST_RECONFIGURE_FAILED: _("Reconfiguring of the %(systype)s "
                               "%(hostname)s failed.  Refer to the "
                               "/var/log/nova/nova-api.log on the management "
                               "system for more details."),
    HOST_UNCONFIGURE_FAILED: _("Unconfiguring of the %(systype)s "
                               "%(hostname)s failed.  Refer to the "
                               "/var/log/nova/nova-api.log on the management "
                               "system for more details."),
    HOST_SOFTWARE_UPDATING: _("The PowerVC software on %(systype)s "
                              "%(hostname)s is being updated to version "
                              "%(version)s."),
    KVM_LIBVIRT_UNAVAILABLE: _("The libvirt Hypervisor is unavailable for "
                               "use.  Verify the service is started and "
                               "working on the %(systype)s."),
    HOST_INVALID_TIME_SKEW: _("The time difference between the management "
                              "system and the %(systype)s is greater than "
                              "30 minutes.  Verify and correct the time on "
                              "both systems and try to add the host again."),
    HOST_CANNOT_CONNECT_MGMT: _("The %(systype)s cannot connect back to the "
                                "management system using %(myip)s due to a "
                                "firewall or other network configuration "
                                "issue."),
    HOST_NON_ROOT_USER: _("A user with 'root\' privileges must be specified "
                          "for the %(systype)s.  Choose a different user and "
                          "try adding the host again."),
    HOST_NO_SPACE: _("The %(systype)s does not have enough disk space for the "
                     "PowerVC install.  Approximately %(freespace)s MB is "
                     "required.  Free up the necessary disk space, then retry "
                     "adding the host."),
    HOST_INVALID_HOSTNAME: _("The hostname on the %(systype)s is set to "
                             "localhost.  Set the hostname to a valid "
                             "hostname and try adding the host again."),
    HOST_SCP_NOT_FOUND: _("The scp command was not found on the management "
                          "system.  The scp command is needed to transfer "
                          "the PowerVC image to the host.  Either install "
                          "the openssh-clients rpm on the management system "
                          "or manually transfer and install the PowerVC image "
                          "to the host."),
    HOST_NEED_SUDOER: _("Either a user with 'root' privileges or a user in "
                        "the /etc/sudoers file must be specified for the "
                        "%(systype)s.  Choose a different user and try "
                        "adding the host again."),
    HOST_SUDOER_NOPASS: _("When using an SSH Private Key for authentication, "
                          "either a user with 'root' privileges or a user in "
                          "the /etc/sudoers file that doesn't require a "
                          "password must be specified for the %(systype)s.  "
                          "Choose a root user or change the user to not "
                          "require a password on sudo and try adding the "
                          "host again."),
    KVM_ABSOLUTE_PATH: _("The ephemeral disk path '%(path)s' specified is not "
                         "an absolute path.  Please specify a valid absolute "
                         "path and try again."),
    HOST_ADD_INPROGRESS: _("The host %(hostname)s is already in the process "
                           "of being added.  Please wait for the previous "
                           "add operation to finish."),
    HOST_DEL_INPROGRESS: _("The host %(hostname)s is already in the process "
                           "of being removed.  Please wait for the previous "
                           "remove operation to finish."),
    HOST_HMC_FORCE_SWITCH: _("The %(regtype)s %(hostname)s is already being "
                             "managed by the PowerVC management system using "
                             "a Hardware Management Console.  Either remove "
                             "the host from PowerVC or request it to be "
                             "switched to managed using the PowerVM NovaLink "
                             "OS instead."),
    HOST_PVM_NOT_MASTER: _("The %(regtype)s %(hostname)s is currently "
                           "configured with a Hardware Management Console as "
                           "the master.  Either release the HMC as the master "
                           "or request it to be switched automatically by "
                           "PowerVC."),
    HOST_PVM_FORCE_SWITCH: _("The %(regtype)s %(hostname)s is already being "
                             "managed by the PowerVC management system using "
                             "a PowerVM NovaLink OS.  Either remove the host "
                             "from PowerVC or request it to be switched to "
                             "managed using the Hardware Management Console "
                             "instead."),
    HOST_HMC_NOT_MASTER: _("The %(regtype)s %(hostname)s could not be added "
                           "or edited since the HMC is not the master for "
                           "this host.  Make sure this HMC is the master for "
                           "the host and try the operation again."),
    HOST_HMC_NOT_MASTER2: _("The %(regtype)s %(hostname)s could not be added "
                            "or edited since the host is configured to be "
                            "able to use a PowerVM NovaLink OS.  Either "
                            "remove or use the NovaLink partition to manage "
                            "the host and try the operation again."),
    HOST_PVM_SWITCH_ERROR: _("The %(regtype)s %(hostname)s could not be "
                             "switched to using the PowerVM NovaLink OS as "
                             "the master.  Try manually changing the master "
                             "for the host and try the operation again."),
    HOST_PVM_NO_SSP: _("The host could not be switched to use PowerVM "
                       "NovaLink since there are virtual machines managed "
                       "using shared storage pools.  Unmanage these virtual "
                       "machines and try the operation again."),
    HOST_PVM_NO_SPP: _("The host could not be switched to use PowerVM "
                       "NovaLink since there are virtual machines managed "
                       "using a non-default shared processor pool.  Unmanage "
                       "these virtual machines and try the operation again."),
    HOST_PVM_MIN_VERISON: _("The host could not be added or updated because "
                            "the pvm-novalink package on the PowerVM NovaLink "
                            "partition is at a version less than %(version)s. "
                            "Please update the NovaLink software on your "
                            "NovaLink partition and try the operation again."),
    NFS_INVALID_IP: _("The IP address specified could not be reached."),
    NFS_INVALID_MOUNT_OPTIONS: _("The mount options specified are invalid."),
    NFS_INVALID_SHARE: _("The NFS share provided is invalid."),
    NFS_UNKNOWN_MOUNT_ERROR: _("The specified NFS mount command is invalid."),
    PVM_NEED_PVMADMIN: _("Either a user with 'root' privileges or a user in "
                         "the 'pvm_admin' group must be specified for the "
                         "%(systype)s.  Add the user to the 'pvm_admin' group "
                         "and try adding the host again.")
}
