#
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from __future__ import print_function
import tools.k2aclient.k2asample as k2asample
from powervc_cinder.k2aclient import client

import logging

# def ssp_get_lu_from_udid(ssp, lu_udid):
#     found = False
#     for lu in ssp.logical_units.logical_unit:
#         if lu.unique_device_id == lu_udid:
#             found = True
#             break
#     if not found:
#         x = "LU w/ udid: >%s<, not found in cluster: >%s<"
#         raise ValueError(x % (lu_udid, cluster_id))
#     return lu


def ssp_lu_clone_exercise(cs, cluster_id):
    """lu_clone sample for k2aclient"""
#
    cluster = cs.cluster.get(cluster_id)
    cdict = {}

    # SOURCEs
    # THICK,DISK
    (status, source_udid_thick_disk, job_id) = cluster.lu_create(
        "P2Z-SOURCE-THICK-DISK", 1,
        thin=False,
        logicalunittype="VirtualIO_Disk",)
    cdict[source_udid_thick_disk] = "P2Z-SOURCE-THICK-DISK"
    # THIN,DISK
    (status, source_udid_thin_disk, job_id) = cluster.lu_create(
        "P2Z-SOURCE-THIN-DISK", 1,
        thin=True,
        logicalunittype="VirtualIO_Disk",)
    cdict[source_udid_thin_disk] = "P2Z-SOURCE-THIN-DISK"
    # THICK,IMAGE
    (status, source_udid_thick_image, job_id) = cluster.lu_create(
        "P2Z-SOURCE-THICK-IMAGE", 1,
        thin=False,
        logicalunittype="VirtualIO_Image",)
    cdict[source_udid_thick_image] = "P2Z-SOURCE-THICK-IMAGE"
    # THIN,IMGAGE
    (status, source_udid_thin_image, job_id) = cluster.lu_create(
        "P2Z-SOURCE-THIN-IMAGE", 1,
        thin=True,
        logicalunittype="VirtualIO_Image",)
    cdict[source_udid_thin_image] = "P2Z-SOURCE-THIN-IMAGE"

    # CLONES
    # CREATE TARGET AND CLONE IN ONE STEP
    cases = [(source_udid_thick_disk, "P2Z-TARGET-THICK-IMAGE", False,
              "VirtualIO_Image"),
             (source_udid_thick_disk, "P2Z-TARGET-THIN-IMAGE", True,
              "VirtualIO_Image"),
             (source_udid_thin_disk, "P2Z-TARGET-THICK-IMAGE", False,
              "VirtualIO_Image"),
             (source_udid_thin_disk, "P2Z-TARGET-THIN-IMAGE", True,
              "VirtualIO_Image"),

             (source_udid_thick_image, "P2Z-TARGET-THICK-DISK", False,
              "VirtualIO_Disk"),
             (source_udid_thick_image, "P2Z-TARGET-THIN-DISK", True,
              "VirtualIO_Disk"),
             (source_udid_thin_image, "P2Z-TARGET-THICK-DISK", False,
              "VirtualIO_Disk"),
             (source_udid_thin_image, "P2Z-TARGET-THIN-DISK", True,
              "VirtualIO_Disk"),

             (source_udid_thick_disk, "P2Z-TARGET-THICK-DISK", False,
              "VirtualIO_Disk"),
             (source_udid_thick_disk, "P2Z-TARGET-THIN-DISK", True,
              "VirtualIO_Disk"),
             (source_udid_thin_disk, "P2Z-TARGET-THICK-DISK", False,
              "VirtualIO_Disk"),
             (source_udid_thin_disk, "P2Z-TARGET-THIN-DISK", True,
              "VirtualIO_Disk"),

             (source_udid_thick_image, "P2Z-TARGET-THICK-IMAGE", False,
              "VirtualIO_Image"),
             (source_udid_thick_image, "P2Z-TARGET-THIN-IMAGE", True,
              "VirtualIO_Image"),
             (source_udid_thin_image, "P2Z-TARGET-THICK-IMAGE", False,
              "VirtualIO_Image"),
             (source_udid_thin_image, "P2Z-TARGET-THIN-IMAGE", True,
              "VirtualIO_Image")]

    for i, (csource, cname, cthin, ctype) in enumerate(cases):

        print ("Attempt >%d<: >%s< ->  >%s<" % (i + 1, cdict[csource], cname))

        try:
            status, dest_lu_udid, job_id = cluster.lu_create(
                cname,
                1,
                thin=cthin,
                logicalunittype=ctype,
                clonedfrom=source_udid_thin_disk)

            if status != "COMPLETED_OK":
                print ("    attempt: >%d< of >%s< -> >%s<"
                       " failed w/ status of: >%s< " %
                       (i + 1, cdict[csource], cname, status))
        except Exception as e:
            print ("    attempt: >%d< of >%s< -> >%s<"
                   " failed w/ exception of: >%s<" %
                   (i + 1, cdict[csource], cname, e))


if __name__ == '__main__':

    k2acfg = k2asample.getk2acfg()
    k2asample.configure_logging(logging.getLevelName(k2acfg['loglevel']))
#     cs = client.Client(k2acfg['api_version'],
#                        "9.114.181.229",  # k2acfg['k2_url'],
#                        k2acfg['k2_username'],
#                        "Passw0rd",  # k2acfg['k2_password'],
#                        k2_auditmemento=k2acfg['k2_auditmemento'],
#                        k2_certpath=k2acfg['k2_certpath'],
#                        retries=k2acfg['retries'],
#                        timeout=k2acfg['timeout'],
#                        excdir=k2acfg['excdir'])

#     # gerald 168
#     k2_url = "9.114.181.168"
#     k2_password = "passw0rd"
#     cluster_id = "02803f50-7063-3602-a304-fb54e4ca2d44"  # p730_810_A

#     # gerald_225
#     k2_url = "9.114.181.225"
#     k2_password = "Passw0rd"
# #     cluster_id = "04628d39-67df-3047-b90e-c4d9b4057267"  # p730_810_A
#     cluster_id = "f598ac8f-c644-36c0-8a4c-1c58809527e0"  # p730_810_B

#     # Gerald
#     k2_url = "9.114.181.238"
#     k2_password = "Passw0rd"
#     cluster_id = "04628d39-67df-3047-b90e-c4d9b4057267"  # carl's new cluster

# #     REJY
#     k2_url = "9.126.139.241"
#     k2_password = k2acfg['k2_password']
#     cluster_id = "c43fbdcd-95f2-3b4a-b643-234ff00eded4"  # TestCluster

# #     N8
#     k2_url = "hmc4.watson.ibm.com"
#     k2_password = k2acfg['k2_password']
#     cluster_id = "0c737495-d09a-337a-a7e9-6173d4bb6d20"  # cluster-c

#     # N7
#     k2_url = "hmc4.watson.ibm.com"
#     k2_password = k2acfg['k2_password']
#     cluster_id = "fe3fbe0f-5ba8-3374-ab75-7b653c9a57ff"  # cluster-b

#     # N3
#     k2_url = "hmc4.watson.ibm.com"
#     k2_password = k2acfg['k2_password']
# #     cluster_id = "fe3fbe0f-5ba8-3374-ab75-7b653c9a57ff"  # cluster-b
#     cluster_id = "8869b443-6566-3184-adfc-066ffb53d0a6"  # cluster-s

    # GFM's 4Q2014 Dev machine
    k2_url = "9.114.181.168"
    k2_password = "abc1234"
    cluster_id = "6f9f0540-d547-32a9-ba3a-ee70d1d81fc8"  # powervc_demo_cluster

    cs = client.Client(k2acfg['api_version'],
                       k2_url,  # k2acfg['k2_url'],
                       k2acfg['k2_username'],
                       k2_password,  # k2acfg['k2_password'],
                       k2_auditmemento=k2acfg['k2_auditmemento'],
                       k2_certpath=k2acfg['k2_certpath'],
                       retries=k2acfg['retries'],
                       timeout=k2acfg['timeout'],
                       excdir=k2acfg['excdir'])

    # sde-power clustercd79f607-671e-3a80-9862-63d48702729e
#     cluster_id = 'c38db666-ba8c-35d0-9fa6-d2bf0cc5ea4b'  # cluster-e
#     cluster_id = '6a948d4c-000f-3219-8972-818c37a94d97'  # cluster-d
#     cluster_id = 'cd79f607-671e-3a80-9862-63d48702729e'  # carl's cluster
#     cluster_id = "ea1b0b5f-3b3a-39dc-bade-6e9cebd18bb2"  # cluster-a
    try:
        ssp_lu_clone_exercise(cs, cluster_id)
    except Exception as e:
        logging.exception(e)
